﻿using System;
using System.Threading;
using System.Windows.Forms;

namespace CSharp411
{
	public partial class Form1 : Form
	{
		#region CONSTRUCTOR
		public Form1()
		{
			this.InitializeComponent();
		}
		#endregion CONSTRUCTOR
		
		#region CLOSE
		delegate void CloseMethod( Form form );
		static private void CloseForm( Form form )
		{
			if (!form.IsDisposed)
			{
				if (form.InvokeRequired)
				{
					CloseMethod method = new CloseMethod( CloseForm );
					form.Invoke( method, new object[] { form }  );
				}
				else
				{
					form.Close();
				}
			}
		}
		static public void CloseAllForms()
		{
			// get array because collection changes as we close forms
			Form[] forms = OpenForms;

			// close every open form
			foreach (Form form in forms)
			{
				CloseForm( form );
			}
		}
		#endregion CLOSE

		#region EVENTS
		private void Button_CloseAll_Click( object sender, EventArgs e )
		{
			CloseAllForms();
		}
		private void Button_NewThis_Click( object sender, EventArgs e )
		{
			OpenForm();
		}
		private void Button_NewThread_Click( object sender, EventArgs e )
		{
			ThreadStart threadStart = new ThreadStart( OpenFormNewThread );
			Thread newThread = new Thread( threadStart );
			newThread.Start();
		}
		#endregion EVENTS

		#region OPEN
		public static Form[] OpenForms
		{
			get
			{
				Form[] forms = null;
				int count = Application.OpenForms.Count;
				forms = new Form[count];
				if (count > 0)
				{
					int index = 0;
					foreach (Form form in Application.OpenForms)
					{
						forms[index++] = form;
					}
				}
				return forms;
			}
		}
		static private void OpenForm()
		{
			Form form = new Form();
			form.Text = Guid.NewGuid().ToString();
			form.Show();
		}
		static private void OpenFormNewThread()
		{
			Form form = new Form();
			form.Text = Guid.NewGuid().ToString();
			form.ShowDialog();
		}
		#endregion OPEN
	}
}
